@@ -1,5 +1,7 @@
 /*
- * Copyright (c) 1991-2003 Kyoto University
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -33,10 +35,10 @@ static int nseq;		/* $BF~NOC18l?t(B */
 /* $B%Q!<%8%s%07k2L(B */
 static STATECHAIN *reach_state;	/* $B:GBgE~C#>uBV(B */
 static boolean can_accept;	/* $B<uM}2DG=$+$I$&$+(B */
-static nseq_reached;		/* $B:GBgE~C#C18l?t(B */
+static int nseq_reached;	/* $B:GBgE~C#C18l?t(B */
 
 /* $BE~C#$7$?>uBV$O%9%?%C%/$K$?$a$F$*$/(B */
-static
+static void
 push_state(int stateid)
 {
   STATECHAIN *new;
@@ -69,7 +71,7 @@ put_state(int s, int l)
 
 /* $B>uBV(Bstateid$B$K$F(Biseq$BHVL\$NF~NO$,<u$1IU$1$i$l$k$+$I$&$+$rJV$9(B */
 /* $B?<$5M%@hC5:w(B */
-static 
+static void
 can_accept_recursive(int stateid, int iseq)
 {
   WTOKEN *token;
@@ -245,9 +247,6 @@ int main(int argc, char *argv[])
   int i, len;
   char *prefix;
   char *dfafile, *dictfile, *termfile;
-#ifdef HAVE_READLINE
-  HIST_ENTRY *last_h;
-#endif
   char *spname_default = SPNAME_DEF;
   char *spname = NULL;
 #define NEXTARG (++i >= argc) ? (char *)usage(argv[0]) : argv[i]
@@ -343,7 +342,7 @@ int main(int argc, char *argv[])
 
   /* initialize readline */
 #ifdef HAVE_READLINE
-  rl_bind_key(PAGE, rl_menu_complete);
+  /* rl_bind_key(PAGE, rl_menu_complete); */
   if (term_mode && !no_term_file) {
 #ifdef HAVE_READLINE_4_1_OLDER
     rl_completion_entry_function = (Function *)dfaterm_generator;
