@@ -1,41 +1,81 @@
+/**
+ * @file   adin_mic_sol2.c
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 19:06:46 2005
+ * 
+ * <JA>
+ * @brief  ޥ (Solaris2.x)
+ *
+ * Solaris 2.x ǥޥϤѤ뤿٥벻ϴؿǤ
+ * Solaris 2.x ΥޥǤϥǥեȤǤΥե뤬Ѥޤ
+ *
+ * Sun Solaris 2.5.1  2.6 ưǧ򤷤Ƥޤ
+ * ӥåǥȤƤ뤿ᡤSolaris x86 Ǥưޤ
+ *
+ * ư奪ǥϤϥޥ˼ưŪڤؤޤ
+ * ܥ塼ϼưĴᤵޤgaintoolʤɤĴᤷƤ 
+ *
+ * ǥեȤΥǥХ̾ "/dev/audio" ǤĶѿ AUDIODEV 
+ * ǥХ̾ꤹ뤳Ȥǡ¾ΥǥХ̾ѤǤޤ
+ * </JA>
+ * <EN>
+ * @brief  Microphone input on Solaris 2.x
+ *
+ * Low level I/O functions for microphone input on Solaris 2.x machines.
+ * This file is used as default on Solaris 2.x machines.
+ *
+ * Tested on Sun Solaris 2.5.1 and 2.6.  Also works on later versions.
+ * Please note that this will not work on Solaris x86, since machine
+ * byte order is fixed to big endian.
+ *
+ * The microphone input device will be automatically selected by Julius
+ * on startup.  Please note that the recoding volue will not be
+ * altered by Julius, and appropriate value should be set by another tool
+ * such as gaintool.
+ * 
+ * The default device name is "/dev/audio", which can be changed by setting
+ * environment variable AUDIODEV.
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* adin_mic_sol2.c --- adin microphone library for Sun SPARC Solaris 2.x */
-
-/* $Id: adin_mic_sol2.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
-/* Tested on Solaris-2.6 */
-/* may not work in Solaris x86 because assuming BIG ENDIAN here */
-
-/* volume will be set to J_DEF_VOLUME at startup */
-
-/* #define J_DEF_VOLUME 14 */		/* 0-99 */
-
 #include <sent/stddefs.h>
 #include <sent/adin.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <unistd.h>
 #include <errno.h>
 #include <sys/stropts.h>
 
 /* sound header */
 #include <sys/audioio.h>
-static int afd;			/* audio file descriptor */
-static struct audio_info ainfo;	/* audio format info */
+static int afd;			///< Audio file descriptor
+static struct audio_info ainfo;	///< Audio format information
 
-/* check audio port resource and initialize */
-/* will be called once at startup time */
+/// Default device name, can be overridden by AUDIODEV environment variable
+#define DEFAULT_DEVICE "/dev/audio"
+
+/** 
+ * Device initialization: check device capability and open for recording.
+ * 
+ * @param sfreq [in] required sampling frequency.
+ * @param arg [in] a dummy data
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_standby(int sfreq, void *arg)
 {
-  char *defaultdev = "/dev/audio"; /* default device file */
+  char *defaultdev = DEFAULT_DEVICE;
   char *devname;
 
   /* get device name if specified in $AUDIODEV */
@@ -99,7 +139,11 @@ adin_mic_standby(int sfreq, void *arg)
 
 }
 
-/* start recording */
+/** 
+ * Start recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_start()
 {
@@ -116,7 +160,11 @@ adin_mic_start()
   return(TRUE);
 }
 
-/* stop recording */
+/** 
+ * Stop recording.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 adin_mic_stop()
 {
@@ -132,8 +180,18 @@ adin_mic_stop()
   return(TRUE);
 }
 
-/* read samples from audio device */
-/* try to read `sampnum' samples and returns actual sample num recorded */
+/**
+ * @brief  Read samples from device
+ * 
+ * Try to read @a sampnum samples and returns actual number of recorded
+ * samples currently available.  This function will block until
+ * at least some samples are obtained.
+ * 
+ * @param buf [out] samples obtained in this function
+ * @param sampnum [in] wanted number of samples to be read
+ * 
+ * @return actural number of read samples, -2 if an error occured.
+ */
 int
 adin_mic_read(SP16 *buf, int sampnum)
 {
