@@ -1,50 +1,68 @@
+/**
+ * @file   paramselect.c
+ * @author Akinobu LEE
+ * @date   Sun Feb 13 20:46:39 2005
+ *
+ * <JA>
+ * @brief  ѥ᡼٥ȥηΥåĴ
+ *
+ * %HMMħѥ᡼ηåޤפפʤ硤
+ * ħѥ᡼ΰ뤳Ȥǰפ褦ĴǤ뤫ɤ
+ * ߤޤ㡧ħ̥ե뤬 MFCC_E_D_Z (26) Ϳ줿
+ * Ȥǥ뤬 MFCC_E_D_N_Z (25) Ǥ硤ͥѥ
+ * ȤĴǤޤ
+ *
+ * Ĵ르ꥺϰʲΤȤǤ
+ *    -# Ϥγƥ٥ȥǤбޡ 0 ˽
+ *    -# %HMM׵ᤵƤ뷿бʤ٥ȥǤ 1 ޡ
+ *    -# ˥ѥ᡼ΰݤɬפǡʥޡƤʤǡ
+ *       Τߤ򥳥ԡ롥
+ * 
+ * </JA>
+ * <EN>
+ * @brief  Check and adjust parameter vector types
+ *
+ * This file is to check if %HMM parameter and input parameter are the same.
+ * If they are not the same, it then tries to modify the input to match the
+ * required format in %HMM.  Available parameter modification is only to
+ * delete some part of the parameter (ex. MFCC_E_D_Z (26 dim.) can be
+ * modified to MFCC_E_D_N_Z (25 dim.) by just deleting the absolute power).
+ * Note that no parameter generation or conversion is implemented currently.
+ *
+ * The adjustment algorithm is as follows:
+ *    -# Initialize mark to 0 for each input vector element.
+ *    -# Compare parameter type and mark unnecessary element as EXCLUDE(=1).
+ *    -# Allocate a new parameter area and copy needed (=NOT marked) element.
+ * </EN>
+ *
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* paramselect.c --- select which parameter component to use */
-
-/* $Id: paramselect.c,v 1.3 2003/09/29 06:01:23 ri Exp $ */
-
-/*                   example1: MFCC_E_D_Z (26)-> MFCC_E_D_N_Z (25)*/
-/*                   example2: MFCC_E_D_A (39)-> MFCC_E_D (26) */
-/* NOTE: can only exclude part of vectors, can't generate new one */
-
 #include <sent/stddefs.h>
 #include <sent/htk_param.h>
 #include <sent/htk_hmm.h>
 
-/*
-   F_ENERGY     0x0040      _E log energy coef. 
-   F_ENERGY_SUP 0x0080      _N (with _E) suppress absolute energy 
-   F_DELTA      0x0100      _D delta (first-order regression) coef. 
-   F_ACCL       0x0200      _A (with _D) acceleration (second-order) coef. 
-   F_COMPRESS   0x0400      _C compressed 
-   F_CEPNORM    0x0800      _Z cepstral mean normalization 
-   F_CHECKSUM   0x1000      _K CRC checksum added 
-   F_ZEROTH     0x2000      _0 (with MFCC) 0'th cepstral parameter
-   */
-
-
-
-/* selection algorithm --- mark-based: */
-/* 1. initialize (=0) mark for each vector element */
-/* 2. compare parameter type which is present and needed, and */
-/*    mark each element which is not needed as EXCLUDE(=1) */
-/* 3. allocate new parameter area and copy only NOT marked element */
-/* '-DDEBUG' displays the process on stdout */
 
 /* vector mark & delete */
-static int *vmark;		/* mark vector */
-static int vlen;		/* length of above */
-static int vnewlen;		/* selected new length */
+static int *vmark;		///< Exclusion marks of input vector
+static int vlen;		///< Length of above
+static int vnewlen;		///< Adjusted new length
+static short src_type;		///< Newly set source type
 
-/* initialize vmark */
+/** 
+ * Allocate new area for exclusion marks.
+ * 
+ * @param param [in] input parameter
+ */
 static void
-init_mark(param)
-     HTK_Param *param;
+init_mark(HTK_Param *param)
 {
   int i;
   vmark = (int *)mymalloc(sizeof(int) * param->veclen);
@@ -53,18 +71,25 @@ init_mark(param)
     vmark[i] = 0;
   }
 }
-/* free vmark */
+
+/** 
+ * Free the exclusion marks.
+ * 
+ */
 static void
 free_mark()
 {
   free(vmark);
 }
 
-/* mark part of vectors ([loc..(loc+len-1)] as exclude) */
+/** 
+ * Put exlusion marks for vector for @a len elements from @a loc -th dimension.
+ * 
+ * @param loc [in] beginning dimension to mark
+ * @param len [in] number of dimension to mark from @a loc
+ */
 static void
-mark_exclude_vector(loc, len)
-     int loc;
-     int len;
+mark_exclude_vector(int loc, int len)
 {
   int i;
 #ifdef DEBUG
@@ -89,14 +114,22 @@ mark_exclude_vector(loc, len)
 #endif
 }
 
-/* execute exclusion */
-/* copy vectors from src to new according to vmark status */
+/**
+ * @brief  Execute exclusion for a parameter data according to the
+ * exclusion marks.
+ * 
+ * Execute vector element exclusion by copying vector from @a src to @a new
+ * according to the current exclusion marks.  The parameter vector of @a new
+ * will be newly allocated here.
+ * 
+ * @param new [out] new adjusted parameter
+ * @param src [in] source parameter
+ */
 static void
-exec_exclude_vectors(new, src)
-     HTK_Param *new;
-     HTK_Param *src;
+exec_exclude_vectors(HTK_Param *new, HTK_Param *src)
 {
-  int i,loc, t;
+  int i, loc;
+  unsigned int t;
   /* src ---(vmark)--> new */
 
   /* new length */
@@ -121,8 +154,16 @@ exec_exclude_vectors(new, src)
   }
 }
 
-/* for in-line one-vector execution */
-/* modify the vector, not copy */
+/** 
+ * @brief  Execute deletion for one vector according to the exlusion marks.
+ *
+ * This can be used to frame-synchronous parameter adjustment.
+ * 
+ * @param vec [I/O] target vector
+ * @param len [in]  length of above
+ * 
+ * @return the new length.
+ */
 int
 exec_exclude_one_vector(VECT *vec, int len)
 {
@@ -140,7 +181,16 @@ exec_exclude_one_vector(VECT *vec, int len)
 
 
 
-/* guess base parameter vector length */
+
+/** 
+ * Guess the length of the base coefficient according to the total vector
+ * length and parameter type.
+ * 
+ * @param p [in] parameter data
+ * @param qualtype [in] parameter type
+ * 
+ * @return the guessed size of the base coefficient.
+ */
 int
 guess_basenum(HTK_Param *p, short qualtype)
 {
@@ -162,10 +212,17 @@ guess_basenum(HTK_Param *p, short qualtype)
   return(size);
 }
 
-/* compare source param type and required type in HTK format, and set mark */
 /* can add: _N */
 /* can sub: _E_D_A_0 */
-static short src_type;
+
+/** 
+ * Compare source parameter type and required type in HTK %HMM, and set mark.
+ * 
+ * @param src [in] input parameter
+ * @param dst_type_arg [in] required parameter type
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 select_param_vmark(HTK_Param *src, short dst_type_arg)
 {
@@ -312,8 +369,15 @@ select_param_vmark(HTK_Param *src, short dst_type_arg)
 }
 
 
-/* extracts needed parameter vector specified in dst_type_arg from src, and */
-/* returns newly allocated parameter structure */
+/** 
+ * Extracts needed parameter vector specified in dst_type_arg from src,
+ * and returns newly allocated parameter structure.
+ * 
+ * @param src [in] input parameter
+ * @param dst_type_arg [in] required parameter type
+ * 
+ * @return newly allocated adjusted parameter, NULL on failure.
+ */
 HTK_Param *
 new_select_param_kind(HTK_Param *src, short dst_type_arg)
 {
@@ -341,18 +405,33 @@ new_select_param_kind(HTK_Param *src, short dst_type_arg)
   return(new);
 }
 
-/* top function: */
-/* compare types for given parameter 'param' and HMM definition 'hmminfo' */
-/* if type is not same, try to adjust by vector selection */
-/* vflag is verbose flag (if TRUE, output verbose messages) */
-/* returns NULL on paramtype error */
+/** 
+ * @brief  Top function to adjust parameter.
+ *
+ * It compares the types for the given parameter @a param and
+ * %HMM definition @a hmminfo.  If type is not the same, adjustment will be
+ * tried.
+ *
+ * If adjustment is performed, the input @a param will be freed and
+ * the newly allocated adjusted parameter will be returned.  Otherwise, 
+ * the original @a param will be returned.
+ * 
+ * @param hmminfo [in] HTK %HMM definition
+ * @param param [i/o] input parameter, will be freed if adjustment was
+ * performed in this function
+ * @param vflag [in] if TRUE, output verbose messages
+ * 
+ * @return the original @a param if no adjustment needed, newly allocated param
+ * is adjustment was needed and successfully done, NULL on failure (in case
+ * parameter type does not match even by the adjustment).
+ */
 HTK_Param *
 new_param_check_and_adjust(HTK_HMM_INFO *hmminfo, HTK_Param *param, boolean vflag)
 {
   HTK_Param *selected_param;
   char pbuf[80],hbuf[80];
   
-  param_code2str(pbuf, param->header.samptype & ~(F_COMPRESS | F_CHECKSUM), FALSE);
+  param_code2str(pbuf, (short)(param->header.samptype & ~(F_COMPRESS | F_CHECKSUM)), FALSE);
   param_code2str(hbuf, hmminfo->opt.param_type, FALSE);  
   if (!check_param_basetype(hmminfo, param)) {
     /* error if base type not match */
