@@ -1,15 +1,39 @@
+/**
+ * @file   rdparam.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 00:16:44 2005
+ *
+ * <JA>
+ * @brief  HTK ħѥ᡼եɤ߹ߡ¤Τοդ
+ *
+ * ħѥ᡼եΥХȥ big endian ꤷƤޤ
+ *  little endian ξǤ¤꼫ưȽ̤ɤ߹ߤޤ
+ *
+ * եħѥ᡼ "_C" ʰ̥ǡˡ"_K" CRCå
+ * ĤˤޤޤƤ硤Ϥǽޤξ硤
+ * ɤ߹ޤ줿ħѥ᡼ǡη餳ϼޤ
+ * </JA>
+ * <EN>
+ * @brief  Read HTK parameter file
+ *
+ * The byte order of HTK parameter file is assumed as big endian.  If not,
+ * however, these functions try to read with forcing byte (re-)swapping.
+ *
+ * When "_C" (compressed) or "_K" (CRC checksum added) exists in the file,
+ * they are processed in these functions.  Then, after reading finished,
+ * these qualifiers are removed from its parameter type code.
+ * </EN>
+ *
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdparam.c --- read in HTK format parameter file */
-
-/* $Id: rdparam.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
-/* _C (compressed) and _K (CRC checksum added) is also handled here */
-
 /* assume sizeof: */
 /*            float = 4 */
 /* (unsigned)   int = 4 */
@@ -19,15 +43,24 @@
 #include <sent/stddefs.h>
 #include <sent/htk_param.h>
 #include <sys/types.h>
-#include <sys/uio.h>
 
-static boolean needswap;	/* TRUE if samples need byte-swapping */
+static boolean needswap;	///< TRUE if need byte-swapping
 
+/** 
+ * Read binary data from a file pointer, with byte swapping.
+ * 
+ * @param buf [out] buffer to store read data
+ * @param unitbyte [in] size of a unit in bytes
+ * @param unitnum [in] number of unit to read
+ * @param fp [in] file pointer
+ * 
+ * @return TRUE if specified number of unit was successfully read, FALSE if failed.
+ */
 static boolean
 myread(char *buf, size_t unitbyte, int unitnum, FILE *fp)
 {
-  int tmp;
-  if ((tmp = myfread(buf, unitbyte, unitnum, fp)) == 0) {
+  size_t tmp;
+  if ((tmp = myfread(buf, unitbyte, unitnum, fp)) < (size_t)unitnum) {
     perror("Error: cannot read\n");
     return(FALSE);
   }
@@ -36,13 +69,19 @@ myread(char *buf, size_t unitbyte, int unitnum, FILE *fp)
   return(TRUE);
 }
 
-/* top function: */
-/* read in HTK parameter file from file handle 'fd' and store to 'pinfo' */
-/* returns TRUE on success, FALSE on error */
-boolean
+
+/** 
+ * Read in a HTK parameter file from @a fp .
+ * 
+ * @param fp [in] file pointer
+ * @param pinfo [in] parameter data to store the read informations
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
+static boolean
 read_param(FILE *fp, HTK_Param *pinfo)
 {
-  int i;
+  unsigned int i;
   int v;
   float *a = NULL, *b = NULL;
   char *buf = NULL; /* for uncompressing */
@@ -149,34 +188,47 @@ read_param(FILE *fp, HTK_Param *pinfo)
 
 }
 
-/* returns newly allocated HTK parameter structure */
+/** 
+ * Allocate a new parameter.
+ * 
+ * @return pointer to the newly allocated area.
+ */
 HTK_Param *
 new_param()
 {
   HTK_Param *ret;
   ret = (HTK_Param *)mymalloc(sizeof(HTK_Param));
+  ret->parvec = NULL;
   ret->samplenum = 0;
   return(ret);
 }
 
-/* free HTK parameter structure */
+/** 
+ * Free the HTK parameter structure.
+ * 
+ * @param pinfo [in] parameter data to be destroyed.
+ */
 void
 free_param(HTK_Param *pinfo)
 {
-  int i;
+  unsigned int i;
   if (pinfo->samplenum > 0) {
-/* 
- *     free(pinfo->parvec[0]);
- */
     for (i=0;i<pinfo->samplenum;i++) {
       free(pinfo->parvec[i]);
     }
-    free(pinfo->parvec);
   }
+  if (pinfo->parvec != NULL) free(pinfo->parvec);
   free(pinfo);
 }
 
-/* read in HTK parameter file 'filename' and store to 'pinfo' */
+/** 
+ * Top function to read a HTK parameter file.
+ * 
+ * @param filename [in] HTK parameter file name 
+ * @param pinfo [in] parameter data (already allocated by new_param())
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 rdparam(char *filename, HTK_Param *pinfo)
 {
