@@ -1,21 +1,48 @@
+/**
+ * @file   init_dfa.c
+ * @author Akinobu LEE
+ * @date   Tue Feb 15 14:20:43 2005
+ * 
+ * <JA>
+ * @brief  DFAʸˡեΥؤɤ߹ߤȥåȥå
+ *
+ * ʸˡե뤫ɤ߹ǧΤ˥åȥåפޤ
+ * DFAʸˡեɤ߹ߤ伭Ȥбդ̵ƥꡦ̵ñ
+ * ФԤޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Load Grammar file into memory and setup
+ *
+ * These functions read a grammar from file and setup for recognition process.
+ * They read a DFA grammar file, make mapping from word dictionary and
+ * find a noise category/word for pause handling.
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* init_dfa.c --- initialize DFA */
-
-/* $Id: init_dfa.c,v 1.6 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/dfa.h>
 #include <sent/vocabulary.h>
 #include <sent/htk_hmm.h>
 
 /* read in dfa info from file */
+/** 
+ * Read in a grammar file and set to DFA grammar structure
+ * 
+ * @param dinfo [i/o] a blank DFA data
+ * @param filename [in] DFA grammar file name
+ */
 void
-init_dfa(DFA_INFO *dinfo, char *filename) /* DFA file -> dinfo */
+init_dfa(DFA_INFO *dinfo, char *filename)
 {
   FILE *fd;
   
@@ -33,11 +60,17 @@ init_dfa(DFA_INFO *dinfo, char *filename) /* DFA file -> dinfo */
   j_printerr("done\n");
 }
 
-/* map words in .dict to terminal symbol in .dfa */
+/** 
+ * Make correspondence between all words in dictionary and categories
+ * in grammar, both from a word to a category and from a category to words.
+ * 
+ * @param dinfo [i/o] DFA grammar, category information will be built here.
+ * @param winfo [i/o] Word dictionary, word-to-category information will be build here.
+ */
 void
 make_dfa_voca_ref(DFA_INFO *dinfo, WORD_INFO *winfo)
 {
-  int i;
+  WORD_ID i;
   boolean okflag = TRUE;
 
   j_printerr("Mapping dict item <-> DFA terminal (category)...");
@@ -60,11 +93,16 @@ make_dfa_voca_ref(DFA_INFO *dinfo, WORD_INFO *winfo)
   j_printerr("done\n");
 }
 
-/* set dfa->sp_id and dfa->is_sp[cate] from hmminfo->sp */
+/** 
+ * Find pause word and pause category information, and set to the grammar data.
+ * 
+ * @param dfa [i/o] DFA grammar, @a sp_id and @a is_sp will be built here.
+ * @param winfo [in] Word dictionary
+ * @param hmminfo [in] HTK %HMM to provide which is short pause %HMM
+ */
 void
 dfa_find_pause_word(DFA_INFO *dfa, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
 {
-  HMM_Logical *sphmm;
   int i, t,p;
   WORD_ID w;
 
@@ -86,6 +124,13 @@ dfa_find_pause_word(DFA_INFO *dfa, WORD_INFO *winfo, HTK_HMM_INFO *hmminfo)
   }
 }
 
+/** 
+ * Append the pause word/category information at the last.
+ * 
+ * @param dst [i/o] DFA grammar
+ * @param src [in] DFA grammar to be appended to @a dst
+ * @param coffset appending category point in @a dst
+ */
 void
 dfa_pause_word_append(DFA_INFO *dst, DFA_INFO *src, int coffset)
 {
@@ -93,7 +138,7 @@ dfa_pause_word_append(DFA_INFO *dst, DFA_INFO *src, int coffset)
   /* dst info must be already appended */
   /* [coffset..dst->term_num-1] is the new categories */
   if (dst->term_num - coffset != src->term_num) {
-    j_exit("InternalError: appended term num not match!\n");
+    j_error("InternalError: appended term num not match!\n");
   }
   
   if (dst->is_sp == NULL) {
