@@ -1,19 +1,36 @@
+/**
+ * @file   rdhmmdef_data.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 01:12:19 2005
+ * 
+ * <JA>
+ * @brief  HTK %HMM եɤ߹ߡ%HMM ǥ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read HTK %HMM definition file: %HMM model
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdhmmdef_data.c --- read in hmmdefs (data = HMM) */
-
-/* $Id: rdhmmdef_data.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 
-extern char *rdhmmdef_token;	/* defined in rdhmmdef.c */
+extern char *rdhmmdef_token;	///< Current token
 
-/* malloc new */
+/** 
+ * Allocate a new data and return it.
+ * 
+ * @return pointer to newly allocated data.
+ */
 HTK_HMM_Data *
 htk_hmmdata_new()
 {
@@ -30,8 +47,13 @@ htk_hmmdata_new()
   return(new);
 }
 
-/* add new to global structure */
-static void
+/** 
+ * Add a new data to the global structure.
+ * 
+ * @param hmm [i/o] %HMM definition data to store it
+ * @param new [in] new data to be added
+ */
+void
 htk_hmmdata_add(HTK_HMM_INFO *hmm, HTK_HMM_Data *new)
 {
   HTK_HMM_Data *match;
@@ -59,7 +81,20 @@ htk_hmmdata_add(HTK_HMM_INFO *hmm, HTK_HMM_Data *new)
   }
 }
 
-/* read in model */
+/** 
+ * @brief  Read one new data and returns the pointer
+ *
+ * If a sub-component of this data is directly defined at here, they
+ * will be read from here and assigned to this data.  If a sub-component
+ * is not defined here but a macro name referencing to the component previously
+ * defined in other place, the data will be searched by the macro name and
+ * the pointer to the found component will be assigned to this model.
+ * 
+ * @param fp [in] file pointer
+ * @param hmm [in] %HMM definition data
+ * 
+ * @return pointer to the newly read data.
+ */
 static HTK_HMM_Data *
 htk_hmmdata_read(FILE *fp, HTK_HMM_INFO *hmm)
 {
@@ -113,13 +148,15 @@ htk_hmmdata_read(FILE *fp, HTK_HMM_INFO *hmm)
   return(new);
 }  
 
-
-/* HMM model definition */
+/** 
+ * Read a new data and store it as a macro.
+ * 
+ * @param name [in] macro name
+ * @param fp [in] file pointer
+ * @param hmm [i/o] %HMM definition data
+ */
 void
-def_HMM(
-	char *name,		/* should not be freed after */
-	FILE *fp,
-	HTK_HMM_INFO *hmm)
+def_HMM(char *name, FILE *fp, HTK_HMM_INFO *hmm)
 {
   HTK_HMM_Data *new;
 
