@@ -1,19 +1,39 @@
+/**
+ * @file   rdhmmdef_tiedmix.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 03:25:11 2005
+ * 
+ * <JA>
+ * @brief  HTK %HMM եɤ߹ߡtied-mixtureǥκʬۥɥ֥å
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Read HTK %HMM definition file: mixture codebook in tied-mixture model
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* rdhmmdef_tiedmix.c --- sub-routine for tied-mixture model */
-
-/* $Id: rdhmmdef_tiedmix.c,v 1.4 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 
-extern char *rdhmmdef_token;	/* defined in rdhmmdef.c */
+extern char *rdhmmdef_token;	///< Current token
 
-/* lookup codebook by name */
+/** 
+ * Look up a data macro by the name.
+ * 
+ * @param hmm [in] %HMM definition data
+ * @param keyname [in] macro name to find
+ * 
+ * @return pointer to the found data, or NULL if not found.
+ */
 static GCODEBOOK *
 codebook_lookup(HTK_HMM_INFO *hmm, char *keyname)
 {
@@ -28,8 +48,13 @@ codebook_lookup(HTK_HMM_INFO *hmm, char *keyname)
   }
 }
 
-/* add new codebook to lookup index */
-static void
+/** 
+ * Add a new data to the global structure.
+ * 
+ * @param hmm [i/o] %HMM definition data to store it
+ * @param new [in] new data to be added
+ */
+void
 codebook_add(HTK_HMM_INFO *hmm, GCODEBOOK *new)
 {
   GCODEBOOK *match;
@@ -46,8 +71,22 @@ codebook_add(HTK_HMM_INFO *hmm, GCODEBOOK *new)
   }
 }
 
-/* make index for GCODEBOOK id -> HTK_HMM_Dens * */
-/* error exit if ~m macro "<name><id>" not found */
+/** 
+ * @brief  Convert codebook ID to the defined %HMM density
+ * 
+ * This function assigns a list of %HMM density definition to the
+ * given codebook.  The densities are searched by the name of
+ * codebook name followed by the mixture component ID starting from 1.
+ * For example, if you have a codebook whose name is "ny4s2m", The densities
+ * of names like "ny4s2m1", "ny4s2m2", ... will be searched through the
+ * %HMM definition data.  The resulting list will be stored in the codebook.
+ *
+ * If some density definitions are not found, they are just skipped.
+ * In this case, a warning message will be output to standard error.
+ * 
+ * @param hmminfo [in] %HMM definition data that has densities
+ * @param book [i/o] codebook, name given and density list will be stored.
+ */
 static void
 tmix_create_codebook_index(HTK_HMM_INFO *hmminfo, GCODEBOOK *book)
 {
@@ -79,9 +118,20 @@ tmix_create_codebook_index(HTK_HMM_INFO *hmminfo, GCODEBOOK *book)
   free(mixname);
 }
 
-/* read in TMIX array */
-/* GCODEBOOK structure will be built from mixture macros when referred from <TMix> directive */
-/* mixture macros that is never referred will be treated as normal mixture */
+/** 
+ * @brief  Read a codebook name and weights, build the codebook structure
+ * on demand, and assigns them to the current state.
+ *
+ * The required codebook on the current token will be assigned to this
+ * state.  If the corresponding codebook structure is not built yet,
+ * it will be constructed here on demand by gathering corresponding mixture
+ * density definitions.  Then this state will store the pointer to the
+ * codebook, together with its own mixture weights in the following tokens.
+ * 
+ * @param fp [in] file pointer
+ * @param state [i/o] current %HMM state to hold pointer to the codebook and their weights
+ * @param hmm [i/o] %HMM definition data, codebook statistics and tied-mixture marker will be modified.
+ */
 void
 tmix_read(FILE *fp, HTK_HMM_State *state, HTK_HMM_INFO *hmm)
 {
@@ -139,7 +189,7 @@ tmix_read(FILE *fp, HTK_HMM_State *state, HTK_HMM_INFO *hmm)
       }
       read_token(fp);
       for(i=0;i<len;i++) {
-	state->bweight[mid] = log(w);
+	state->bweight[mid] = (PROB)log(w);
 	mid++;
       }
     }
