@@ -1,17 +1,34 @@
+/**
+ * @file   ngram_malloc.c
+ * @author Akinobu LEE
+ * @date   Wed Feb 16 16:48:56 2005
+ * 
+ * <JA>
+ * @brief  N-gram¤ΤνݤȲ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Initial memory allocation and free for N-gram stucture
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* ngram_malloc.c -- malloc and free n-gram info */
-
-/* $Id: ngram_malloc.c,v 1.3 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/ngram2.h>
 
-/* allocate new ngram info */
+/** 
+ * Allocate a new N-gram structure.
+ * 
+ * @return pointer to the newly allocated structure.
+ */
 NGRAM_INFO *
 ngram_info_new()
 {
@@ -22,7 +39,11 @@ ngram_info_new()
   return(new);
 }
 
-/* free ngram info */
+/** 
+ * Free N-gram data.
+ * 
+ * @param ndata [in] N-gram data
+ */
 void
 ngram_info_free(NGRAM_INFO *ndata)
 {
@@ -48,8 +69,11 @@ ngram_info_free(NGRAM_INFO *ndata)
   free(ndata->n2tonid);
   free(ndata->p_lr);
   free(ndata->p_rl);
+  free(ndata->n2bo_upper);
+  free(ndata->n2bo_lower);
   free(ndata->bo_wt_rrl);
-  free(ndata->n3_bgn);
+  free(ndata->n3_bgn_upper);
+  free(ndata->n3_bgn_lower);
   free(ndata->n3_num);
   /* free 3-gram */
   free(ndata->n3tonid);
