@@ -1,14 +1,52 @@
+/**
+ * @file   gprune_heu.c
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 05:44:52 2005
+ * 
+ * <JA>
+ * @brief  祬ʬ۷׻: Gaussian pruning (heuristic algorithm)
+ *
+ * gprune_heu()Ϻ祬ʬ۽η׻롼ΰĤǤ
+ * heuristic pruning Ȥäƾ̤ΥʬۤνϳΨΤߤ®˵ޤ
+ * Tied-mixture %HMM ѻ Julius GPRUNE_DEFAULT_HEURISTIC 
+ * Ƥ뤫뤤JuliusΥץ "-gprune heuristic" 
+ * ꤹ뤳ȤǤδؿѤޤ
+ *
+ * heuristic pruning ϤޤȤƤʤˡǡ®θ̤
+ * ¾μˡ֤Ǥ
+ * 
+ * gprune_heu()  outprob_init() ˤäƤδؿؤΥݥ󥿤
+ * compute_gaussset ˥åȤ뤳ȤǻѤޤΥݥ󥿤
+ * calc_tied_mix() ޤ calc_mix() ƤӽФޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Calculate probability of a set of Gaussian densities by
+ * Gaussian pruning: heuristic algorithm
+ *
+ * gprune_heu() is one of the functions to compute output probability of
+ * a set of Gaussian densities.  This function does heuristic pruning, trying
+ * to compute only the best ones for faster computation.  If a tied-mixture
+ * %HMM model with GPRUNE_DEFAULT_HEURISTIC defined in Julius, or explicitly
+ * specified by "-gprune heuristic" option, this function will be used.
+ *
+ * The effect of heuristic pruning is practically a middle of others.
+ *
+ * gprune_heu() will be used by calling outprob_init() to set its pointer
+ * to the global variable @a compute_gaussset.  Then it will be called from
+ * calc_tied_mix() or calc_mix().
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* gprune_heu.c --- calculate probability of Gaussian densities */
-/*                   with Gaussian pruning (heuristic) */
-
-/* $Id: gprune_heu.c,v 1.3 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/htk_hmm.h>
 #include <sent/htk_param.h>
@@ -65,12 +103,15 @@
 */
 
 /* pruning threshold */
-static LOGPROB *backmax;	/* backward sum of max for each dimension (inversed) */
-static int backmax_num;		/* veclen */
+static LOGPROB *backmax;	///< Backward sum of max for each dimension (inversed)
+static int backmax_num;		///< Length of above
 
-static boolean *mixcalced;	/* mark which Gaussian has been computed */
+static boolean *mixcalced;	///< Mark which Gaussian has been computed
 
-/* initialize backmax */
+/** 
+ * Clear backmax information.
+ * 
+ */
 static void
 init_backmax()
 {
@@ -78,7 +119,11 @@ init_backmax()
   for(i=0;i<backmax_num;i++) backmax[i] = 0;
 }
 
-/* sum-up backward to make backmax */
+/** 
+ * Build backmax information for each frame, by summing up
+ * current maximum values of each dimensions.
+ * 
+ */
 /*                        |
  *  0 1 2 ... max-3 max-2 | max-1
  *
@@ -101,7 +146,18 @@ make_backmax()
     }*/
 }
 
-/* calculate probability while setting max values to backmax */
+/**
+ * @brief  Calculate probability with maximum value update.
+ * 
+ * Calculate probability of a Gaussian toward OP_vec,
+ * while storing the maximum values of each dimension to @a backmax.
+ * for future pruning.  The pruning itself is not performed here.
+ * This function will be used to compute the first N Gaussians.
+ * 
+ * @param binfo [in] Gaussian density
+ * 
+ * @return the output log probability.
+ */
 static LOGPROB
 compute_g_heu_updating(HTK_HMM_Dens *binfo)
 {
@@ -126,7 +182,20 @@ compute_g_heu_updating(HTK_HMM_Dens *binfo)
   }
   return((sum + binfo->gconst) / -2.0);
 }
-/* calculate probability with pruning with backmax heuristics */
+
+/** 
+ * @brief  Calculate probability with pruning.
+ * 
+ * Calculate probability of a Gaussian toward OP_vec,
+ * performing pruning using threshold and the backmax information
+ * that has been set by compute_g_heu_updating() and
+ * make_backmax().
+ * 
+ * @param binfo [in] Gaussian density
+ * @param thres [in] threshold 
+ * 
+ * @return the output log probability.
+ */
 static LOGPROB
 compute_g_heu_pruning(HTK_HMM_Dens *binfo, LOGPROB thres)
 {
@@ -157,7 +226,11 @@ compute_g_heu_pruning(HTK_HMM_Dens *binfo, LOGPROB thres)
 }
 
 
-/* init */
+/** 
+ * Initialize and setup work area for Gaussian pruning by heuristic algorithm.
+ * 
+ * @return TRUE on success, FALSE on failure.
+ */
 boolean
 gprune_heu_init()
 {
@@ -174,7 +247,30 @@ gprune_heu_init()
   return TRUE;
 }
 
-/* compute a set of Gaussians with safe pruning */
+/** 
+ * @brief  Compute a set of Gaussians with heuristic pruning.
+ *
+ * If the N-best mixtures in the previous frame is specified in @a last_id,
+ * They are first computed to get the maximum value for each dimension.
+ * After that, the rest of the Gaussians will be computed using the maximum
+ * values as heuristics of uncomputed dimensions to drop unpromising
+ * Gaussians from computation at early stage
+ * of likelihood computation.  If the @a last_id is not specified (typically
+ * at the first frame of the input), a safe pruning as same as one in
+ * gprune_safe.c will be applied.
+ *
+ * The calculated scores will be stored to OP_calced_score, with its
+ * corresponding mixture id to OP_calced_id.  These are done by calling
+ * cache_push().
+ * The number of calculated mixtures is also stored in OP_calced_num.
+ * 
+ * This can be called from calc_tied_mix() or calc_mix().
+ * 
+ * @param g [in] set of Gaussian densities to compute the output probability
+ * @param gnum [in] length of above
+ * @param last_id [in] ID list of N-best mixture in previous input frame,
+ * or NULL if not exist
+ */
 void
 gprune_heu(HTK_HMM_Dens **g, int gnum, int *last_id)
 {
