@@ -1,43 +1,63 @@
+/**
+ * @file   vsegment.c
+ * @author Akinobu LEE
+ * @date   Fri Feb 18 19:29:22 2005
+ * 
+ * <JA>
+ * @brief  ϤФViterbi 饤Ȥμ¹
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Do viterbi alignment for the input
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* vsegment.c --- generic routine to do viterbi segmentation */
-
-/* $Id: vsegment.c,v 1.7 2004/01/13 08:38:02 ri Exp $ */
-
-/* any segmentatino unit is allowed: segmentation point should be specified by sequence of HMM number to pickup */
-
 #include <sent/stddefs.h>
 #include <sent/htk_param.h>
 #include <sent/hmm.h>
 
-typedef struct __seg_token__ {
-  int last_id;
-  int last_end_frame;
-  LOGPROB last_end_score;
-  struct __seg_token__ *next;
-  struct __seg_token__ *list;
-} SEGTOKEN;
-
-LOGPROB				/* returns the acoustic score */
-viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
-	     HTK_Param *param,	/* input parameter vectors */
-	     int *endstates,	/* where unit ends in the hmm */
-	     int ulen,		/* total unit numin the hmm */
-	     int **id_ret,       /* return sequence of result */
-	     int **seg_ret,	/* return segmented frame number at each endstates */
-	     LOGPROB **uscore_ret,/* return normalized scores for each unit */
-	     int *slen_ret)
+/** 
+ * @brief  Perform Viterbi alignment.
+ *
+ * This function performs viterbi alignment for the given sentence %HMM,
+ * input parameter and unit definition.  Any segmentatino unit (word, phoneme
+ * state, etc.) is allowed: the segmentation unit should be specified by
+ * specifying a list of state id which are the end of each unit.
+ * For example, if you want to obtain phoneme alignment, the list of state
+ * number that exist at the end of phones should be specified by @a endstates.
+ * 
+ * @param hmm [in] sentence HMM to be matched
+ * @param param [in] input parameter data
+ * @param endstates [in] list of state id that corrsponds to the ends of units
+ * @param ulen [in] total number of units in the @a hmm
+ * @param id_ret [out] Pointer to store the newly allocated array of the resulting id sequence of units on the best path.
+ * @param seg_ret [out] Pointer to store the newly allocated array of the resulting end frame of each unit on the best path.
+ * @param uscore_ret [out] Pointer to store the newly allocated array of the resulting score at the end frame of each unit on the best path.
+ * @param slen_ret [out] Pointer to store the total number of units on the best path.
+ * 
+ * @return the total acoustic score for the whole input.
+ */
+LOGPROB
+viterbi_segment(HMM *hmm, HTK_Param *param, int *endstates, int ulen, int **id_ret, int **seg_ret, LOGPROB **uscore_ret, int *slen_ret)
 {
   /* for viterbi */
   LOGPROB *nodescore[2];	/* node buffer */
   SEGTOKEN **tokenp[2];		/* propagating token which holds segment info */
   int *from_node;
+#ifndef MULTIPATH_VERSION
   int *u_end, *u_start;	/* the node is an end of the word, or -1 */
-  int i, n, t;
+#endif
+  int i, n;
+  unsigned int t;
   int tl,tn;
   LOGPROB tmpsum;
   A_CELL *ac;
@@ -54,6 +74,7 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
     return LOG_ZERO;
   }
 
+#ifndef MULTIPATH_VERSION
   /* initialize unit start/end marker */
   u_start = (int *)mymalloc(hmm->len * sizeof(int));
   u_end   = (int *)mymalloc(hmm->len * sizeof(int));
@@ -72,6 +93,7 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
     printf("unit %d: start=%d, end=%d\n", i, u_start[i], u_end[i]);
   }
 #endif
+#endif /* MULTIPATH_VERSION */
 
   /* initialize node buffers */
   tn = 0;
@@ -80,6 +102,9 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
   for (i=0;i<2;i++){
     nodescore[i] = (LOGPROB *)mymalloc(hmm->len * sizeof(LOGPROB));
     tokenp[i] = (SEGTOKEN **)mymalloc(hmm->len * sizeof(SEGTOKEN *));
+    for (n = 0; n < hmm->len; n++) {
+      tokenp[i][n] = NULL;
+    }
   }
   for (n = 0; n < hmm->len; n++) {
     nodescore[tn][n] = LOG_ZERO;
@@ -98,10 +123,20 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
   /*if (hmm->state[0].is_pseudo_state) {
     j_printerr("Warning: state %d: pseudo state?\n", 0);
     }*/
+#ifdef MULTIPATH_VERSION
+  nodescore[tn][0] = 0.0;
+#else
   nodescore[tn][0] = outprob(0, &(hmm->state[0]), param);
+#endif
 
   /* do viterbi for rest frame */
-  for (t = 1; t < param->samplenum; t++) {
+  for (
+#ifdef MULTIPATH_VERSION
+       t = 0; t <= param->samplenum; t++
+#else
+       t = 1; t < param->samplenum; t++
+#endif
+       ) {
     i = tl;
     tl = tn;
     tn = i;
@@ -127,6 +162,16 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
     }
     /* propagate token, appending new if path was selected between units */
     for (n = 0; n < hmm->len; n++) {
+#ifdef MULTIPATH_VERSION
+      if (from_node[n] == -1 || nodescore[tn][n] <= LOG_ZERO) {
+	/*tokenp[tn][n] = NULL;*/
+      } else {
+	i=0;
+	while (from_node[n] > endstates[i]) i++;
+	if (n > endstates[i]) {
+	  newtoken = (SEGTOKEN *)mymalloc(sizeof(SEGTOKEN));
+	  newtoken->last_id = i;
+#else
       if (from_node[n] == -1) {
 	tokenp[tn][n] = NULL;
       } else if (nodescore[tn][n] <= LOG_ZERO) {
@@ -136,6 +181,7 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
 	    && from_node[n] !=  n) {
 	  newtoken = (SEGTOKEN *)mymalloc(sizeof(SEGTOKEN));
 	  newtoken->last_id = u_end[from_node[n]];
+#endif
 	  newtoken->last_end_frame = t-1;
 	  newtoken->last_end_score = nodescore[tl][from_node[n]];
 	  newtoken->list = root;
@@ -147,8 +193,17 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
 	}
       }
     }
+
+#ifdef MULTIPATH_VERSION
+    /* if this is next of last frame, loop ends here */
+    if (t == param->samplenum) break;
+#endif
+	
     /* calc outprob to new nodes */
     for (n = 0; n < hmm->len; n++) {
+#ifdef MULTIPATH_VERSION
+      if (hmm->state[n].out.state == NULL) continue;
+#endif
       if (nodescore[tn][n] > LOG_ZERO) {
 	if (hmm->state[n].is_pseudo_state) {
 	  j_printerr("Warning: state %d: pseudo state?\n", n);
@@ -165,6 +220,9 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
     for (i=0;i<ulen;i++) {
       printf("%d: unit %d(%d-%d): begin_frame = %d\n", t - 1, i,
 	     (i > 0) ? endstates[i-1]+1 : 0, endstates[i],
+#ifdef MULTIPATH_VERSION
+	     (tokenp[tl][endstates[i]] == NULL) ? -1 :
+#endif
 	     tokenp[tl][endstates[i]]->last_end_frame + 1);
     }
 #endif
@@ -177,7 +235,11 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
 
   /* parse back the last token to see the trail of best viterbi path */
   /* and store the informations to returning buffer */
+#ifdef MULTIPATH_VERSION
+  slen = 0;
+#else
   slen = 1;
+#endif
   for(token = tokenp[tn][hmm->len-1]; token; token = token->next) {
     if (token->last_end_frame == -1) break;
     slen++;
@@ -186,10 +248,14 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
   seg = (int *)mymalloc(sizeof(int)*slen);
   uscore = (LOGPROB *)mymalloc(sizeof(LOGPROB)*slen);
 
+#ifdef MULTIPATH_VERSION
+  i = slen - 1;
+#else
   id[slen-1] = ulen - 1;
   seg[slen-1] = t - 1;
   uscore[slen-1] = result_score;
   i = slen - 2;
+#endif
   for(token = tokenp[tn][hmm->len-1]; token; token = token->next) {
     if (i < 0 || token->last_end_frame == -1) break;
     id[i] = token->last_id;
@@ -211,8 +277,10 @@ viterbi_segment(HMM *hmm,	/* concatinated sentence HMMs */
   *slen_ret = slen;
 
   /* free memory */
+#ifndef MULTIPATH_VERSION
   free(u_start);
   free(u_end);
+#endif
   free(from_node);
   token = root;
   while(token) {
