@@ -1,21 +1,50 @@
+/**
+ * @file   ss.c
+ * @author Akinobu LEE
+ * @date   Thu Feb 17 17:19:54 2005
+ * 
+ * <JA>
+ * @brief  ڥȥ븺
+ *
+ * ºݤΥڥȥ븺 wav2mfcc-buffer.c  wav2mfcc-pipe.c 
+ * ԤޤǤʿѥڥȥοȥեI/OΤƤޤ
+ * </JA>
+ * 
+ * <EN>
+ * @brief  Spectral subtraction
+ *
+ * The actual subtraction will be performed in wav2mfcc-buffer.c and
+ * wav2mfcc-pipe.c.  These functions are for estimating average spectrum
+ * of audio input, and file I/O for that.
+ * </EN>
+ * 
+ * $Revision:$
+ * 
+ */
 /*
- * Copyright (c) 1991-2003 Kyoto University
- * Copyright (c) 2000-2003 NAIST
+ * Copyright (c) 1991-2005 Kawahara Lab., Kyoto University
+ * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
+ * Copyright (c) 2005      Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* ss.c --- load spectral subtraction parameter */
-
-/* $Id: ss.c,v 1.3 2003/09/29 06:01:23 ri Exp $ */
-
 #include <sent/stddefs.h>
 #include <sent/mfcc.h>
 
+
+/** 
+ * Binary read function with byte swaping (assume file is BIG ENDIAN)
+ * 
+ * @param buf [out] read data
+ * @param unitbyte [in] size of a unit in bytes
+ * @param unitnum [in] number of unit to be read
+ * @param fp [in] file pointer
+ */
 static boolean
 myread(void *buf, size_t unitbyte, int unitnum, FILE *fp)
 {
-  int tmp;
-  if ((tmp = myfread(buf, unitbyte, unitnum, fp)) < unitnum) {
+  size_t tmp;
+  if ((tmp = myfread(buf, unitbyte, unitnum, fp)) < (size_t)unitnum) {
     return(FALSE);
   }
 #ifndef WORDS_BIGENDIAN
@@ -24,7 +53,14 @@ myread(void *buf, size_t unitbyte, int unitnum, FILE *fp)
   return(TRUE);
 }
 
-/* load noise spectrum from file (allocate new buffer) */
+/** 
+ * Load a noise spectrum from file.
+ * 
+ * @param filename [in] path name of noise spectrum file
+ * @param slen [out] length of the returned buffer
+ * 
+ * @return a newly allocated buffer that holds the loaded noise spectrum.
+ */
 float *
 new_SS_load_from_file(char *filename, int *slen)
 {
@@ -34,7 +70,7 @@ new_SS_load_from_file(char *filename, int *slen)
 
   /* open file */
   j_printerr("Reading Noise Spectrum for SS...");
-  if ((fp = fopen(filename, "rb")) == NULL) {
+  if ((fp = fopen_readfile(filename)) == NULL) {
     j_printerr("Error: SS_load_from_file: cannot open \"%s\"\n", filename);
     return(NULL);
   }
@@ -51,14 +87,24 @@ new_SS_load_from_file(char *filename, int *slen)
     return(NULL);
   }
   /* close file */
-  fclose(fp);
+  fclose_readfile(fp);
 
   *slen = num;
   j_printerr("done\n");
   return(sbuf);
 }
 
-/* compute noise spectrum from head silence for each input */
+/** 
+ * Compute average spectrum of audio input.
+ * This is used to estimate a noise spectrum from input samples.
+ * 
+ * @param wave [in] input audio data sequence
+ * @param wavelen [in] length of above
+ * @param para [in] parameter
+ * @param slen [out] length of returned buffer
+ * 
+ * @return a newly allocated buffer that contains the calculated spectrum.
+ */
 float *
 new_SS_calculate(SP16 *wave, int wavelen, Value para, int *slen)
 {
@@ -73,6 +119,12 @@ new_SS_calculate(SP16 *wave, int wavelen, Value para, int *slen)
     fftN *= 2; n++;
   }
 
+#ifdef MFCC_SINCOS_TABLE
+  /* generate tables */
+  make_costbl_hamming(para.framesize);
+  make_fft_table(n);
+#endif
+  
   /* allocate work area */
   bf = (float *)mymalloc((fftN + 1) * sizeof(float));
   Re = (float *)mymalloc((fftN + 1) * sizeof(float));
